/* InterfaceBuilder - IBEditors.h
 * Copyright (c) 1995-1996 NeXT Software, Inc.
 * All rights reserved.
 */

#import <AppKit/AppKit.h>
#import <InterfaceBuilder/IBSystem.h>

@interface NSObject (IBEditorSpecification)
- (NSString *)editorClassName;
@end

@protocol IBSelectionOwners <NSObject>
- (unsigned int)selectionCount; 
- (NSArray *)selection;
- (void)drawSelection;
@end

@protocol IBEditors <IBSelectionOwners>

/* Editing objects. */
- (id)initWithObject:(id)object inDocument:(id /*<IBDocuments>*/)document;
- (void)close;

/* Identifying objects. */
- (id)editedObject;
- (id /*<IBDocuments>*/)document;
- (NSWindow *)window;

/* Displaying objects. */
- (void)resetObject:(id)object;

/* Managing the selection. */
- (BOOL)wantsSelection;
- (void)selectObjects:(NSArray *)objects;
- (void)makeSelectionVisible:(BOOL)showIt;

/* Copying and pasting objects. */
- (BOOL)copySelection;
- (BOOL)deleteSelection;
- (BOOL)pasteInSelection;
- (BOOL)acceptsTypeFromArray:(NSArray *)types;

/* Opening and closing editors. */
- (id <IBEditors>)openSubeditorForObject:(id)object;
- (void)closeSubeditors;

/* Activating the editor. */
- (void)orderFront;
- (BOOL)activate;
- (void)validateEditing;

@end

IB_EXTERN NSString *IBSelectionChangedNotification;
IB_EXTERN NSString *IBAttributesChangedNotification;
