/* InterfaceBuilder - IBResourceManager.h
 * Copyright (c) 1996 NeXT Software, Inc.
 * All rights reserved.
 */

#import <InterfaceBuilder/IBDocuments.h>

@interface IBResourceManager : NSObject
{
    id <IBDocuments> document;
}
+ (void)registerResourceManagerClass:(Class)class;
+ (NSArray *)registeredResourceManagerClasses;

- (id)initWithDocument:(id <IBDocuments>)document;
- (id <IBDocuments>)document;
- (void)writeToDocumentPath:(NSString *)path;

- (NSArray *)resourceFileTypes;
- (NSArray *)resourcePasteboardTypes;
- (BOOL)acceptsResourcesFromPasteboard:(NSPasteboard *)pasteboard;
- (void)addResourcesFromPasteboard:(NSPasteboard *)pasteboard;

/* Only sent if file is in IBProjectResourcesFileKey bucket & file extension
 * is one of resource file types.  If you want to track other project
 * changes, you must register for the appropriate notification yourself. */
- (void)project:(id <IBProjects>)project didAddFile:(NSString *)file;
- (void)project:(id <IBProjects>)project didRemoveFile:(NSString *)file;
- (void)project:(id <IBProjects>)project
	didChangeLocalizationOfFile:(NSString *)file;

/* Called when an application posts the appropriate notification to IB's DO
 * port.  This is a convenience for the IBApplicationDidChangeFileNotification
 * below.  Only sent if the file extension of the path is one of resource
 * file types. */
- (void)application:(NSString *)applicationName
	didModifyFileAtPath:(NSString *)path;
@end

/* Notifications. */

/* Notifcation object is the new IBResourceManager class. */
IB_EXTERN NSString *IBResourceManagerRegistryDidChangeNotification;

/* Sent when an application posts the appropriate notification to IB's DO
 * port.  Notification object is the name of the application; userInfo
 * contains "Path", the path of the changed file. */
IB_EXTERN NSString *IBApplicationDidChangeFileNotification;
